/*
 * Copyright (c) 2019-2020. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.scanner.format;

import java.util.Arrays;

import static org.hedgecode.chess.scanner.format.PGNConstants.*;

/**
 * TypeMovesFormat
 *
 * @author Dmitry Samoshin aka gotty
 */
public enum TypeMovesFormat {

    LINE ( new LineMovesFormat() ),
    WRAP ( new WrapMovesFormat() );

    private MovesFormat movesFormat;

    TypeMovesFormat(MovesFormat movesFormat) {
        this.movesFormat = movesFormat;
    }

    public String format(Move[] moves) {
        return movesFormat.format(moves);
    }

    static class LineMovesFormat extends WrapMovesFormat {

        LineMovesFormat() {
            super(PGN_MAX_LINE_LENGTH);
        }

    }

    static class WrapMovesFormat implements MovesFormat {

        private int lineLength;

        WrapMovesFormat() {
            this(PGN_DEF_LINE_LENGTH);
        }

        WrapMovesFormat(int length) {
            lineLength = length;
        }

        @Override
        public String format(Move[] moves) {
            Arrays.sort(moves);
            int maxLength = lineLength - PGN_CRLF.length();
            int length = 0;
            StringBuilder sb = new StringBuilder();
            for (Move move : moves) {
                String nextMove = move.ply() % 2 != 0
                        ? String.format(WHITE_MOVE_FORMAT, move.ply() / 2 + 1, move.move())
                        : String.format(BLACK_MOVE_FORMAT, move.move());
                if (length + nextMove.length() > maxLength) {
                    sb.append(PGN_CRLF);
                    length = 0;
                }
                sb.append(nextMove);
                length += nextMove.length();
            }
            return sb.toString().trim();
        }

    }

}
