/*
 * Copyright (c) 2019-2020. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.scanner.portal;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
import java.util.Map;

import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;

import org.hedgecode.chess.scanner.ChessHogScannerException;
import org.hedgecode.chess.scanner.Scanner;
import org.hedgecode.chess.scanner.regex.RegexMatcher;
import org.hedgecode.chess.scanner.regex.RegexResult;
import org.hedgecode.chess.scanner.regex.RegexType;
import org.hedgecode.chess.scanner.regex.RegexTypeMatcher;

import static org.hedgecode.chess.scanner.ChessHogScannerConstants.*;

/**
 * AbstractRequestScanner
 *
 * @author Dmitry Samoshin aka gotty
 */
public abstract class AbstractRequestScanner implements Scanner {

    protected String request(String url) throws ChessHogScannerException {
        return request(url, true);
    }

    protected String request(String url, boolean withCrlf) throws ChessHogScannerException {
        HttpGet request = new HttpGet(url);
        CloseableHttpResponse response = null;
        StringBuilder sb = new StringBuilder();
        try {
            response = getClient().execute(request);
            try (BufferedReader br = new BufferedReader(
                    new InputStreamReader(
                            response.getEntity().getContent(), CHARSET
                    )
            )) {
                String line;
                while ((line = br.readLine()) != null) {
                    sb.append(line);
                    if (withCrlf) {
                        sb.append(CRLF);
                    }
                }
            }
        } catch (IOException cause) {
            throw new ChessHogScannerException(
                    String.format("Error occurred for requesting URL: %s", url), cause
            );
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException ignored) {}
            }
        }
        return sb.toString();
    }

    protected List<String> split(String url, String splitMatch) throws ChessHogScannerException {
        RegexResult result = splitRequest(url, splitMatch);
        return result.resultList();
    }

    protected Map<String, String> splitMap(String url, String splitMatch) throws ChessHogScannerException {
        RegexResult result = splitRequest(url, splitMatch);
        return result.resultMap();
    }

    protected RegexResult splitRequest(String url, String splitMatch) throws ChessHogScannerException {
        RegexMatcher matcher = new RegexTypeMatcher(
                RegexType.SPLIT, splitMatch
        );
        return matchRequest(url, matcher, false);
    }

    protected String match(String url, String match) throws ChessHogScannerException {
        List<String> result = match(url, match, true);
        return result.isEmpty() ? null : result.get(0);
    }

    protected List<String> match(String url, String match, boolean isFirst)
            throws ChessHogScannerException
    {
        RegexMatcher matcher = new RegexTypeMatcher(
                RegexType.FIND, match
        );
        RegexResult result = matchRequest(
                url,
                matcher,
                isFirst
        );
        return result.resultList();
    }

    protected Map<String, String> matchMap(String url, String match, boolean isFirst)
            throws ChessHogScannerException
    {
        RegexMatcher matcher = new RegexTypeMatcher(
                RegexType.FIND, match
        );
        RegexResult result = matchRequest(
                url,
                matcher,
                isFirst
        );
        return result.resultMap();
    }

    protected String match(String url, String startMatch, String endMatch)
            throws ChessHogScannerException
    {
        List<String> result = match(url, startMatch, endMatch, true);
        return result.isEmpty() ? null : result.get(0);
    }

    protected List<String> match(String url, String startMatch, String endMatch, boolean isFirst)
            throws ChessHogScannerException
    {
        RegexMatcher matcher = new RegexTypeMatcher(
                startMatch, endMatch
        );
        RegexResult result = matchRequest(
                url,
                matcher,
                isFirst
        );
        return result.resultList();
    }

    protected Map<String, String> matchMap(String url, String startMatch, String endMatch, boolean isFirst)
            throws ChessHogScannerException
    {
        RegexMatcher matcher = new RegexTypeMatcher(
                startMatch, endMatch
        );
        RegexResult result = matchRequest(
                url,
                matcher,
                isFirst
        );
        return result.resultMap();
    }

    protected RegexResult matchRequest(String url, RegexMatcher matcher, boolean isFirst)
            throws ChessHogScannerException
    {
        HttpGet request = new HttpGet(url);
        CloseableHttpResponse response = null;
        try {
            response = getClient().execute(request);
            try (BufferedReader br = new BufferedReader(
                    new InputStreamReader(
                            response.getEntity().getContent(),
                            CHARSET
                    )
            )) {
                String line;
                while ((line = br.readLine()) != null) {
                    matcher.match(line);
                    if (isFirst && !matcher.result().isEmpty()) {
                        break;
                    }
                }
            }
        } catch (IOException cause) {
            throw new ChessHogScannerException(
                    String.format("Error occurred for requesting URL: %s", url), cause
            );
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException ignored) {}
            }
        }
        return matcher.result();
    }

    private CloseableHttpClient getClient() {
        return HttpClients.createMinimal();
    }

}
